"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MQTTHub = void 0;
const mqtt_1 = __importDefault(require("mqtt"));
class MQTTHub {
    constructor(options) {
        this.options = options;
        this.DEVICE_DATA_PREFIX = this.options.exchange || `device_values`;
        this.DEVICE_ONLINE_PREFIX = `device_online`;
    }
    setup() {
        return __awaiter(this, void 0, void 0, function* () {
            // let authSection = this.options.user ? `${this.options.user}:${this.options.pass}` : undefined;
            this.client = mqtt_1.default.connect(this.options.host, {
                username: this.options.user,
                password: this.options.pass
            });
            this.client.on('connect', () => {
                var _a;
                console.log("MQTT Client connected!");
                (_a = this.client) === null || _a === void 0 ? void 0 : _a.subscribe(`${this.DEVICE_DATA_PREFIX}/+/#`, (err) => {
                    if (err) {
                        console.error(`Failed to subscribe to ${this.DEVICE_DATA_PREFIX}`);
                    }
                    else {
                        console.log(`Subscribed to ${this.DEVICE_DATA_PREFIX}`);
                    }
                });
            });
            this.client.on('message', (topic, payload, packet) => {
                var _a, _b, _c, _d, _e, _f;
                if (topic.indexOf(this.DEVICE_DATA_PREFIX) > -1) {
                    let messageContent = JSON.parse((payload === null || payload === void 0 ? void 0 : payload.toString()) || '{error: "No message content"}');
                    const regex = new RegExp(`${this.DEVICE_DATA_PREFIX}/(.+?)/`);
                    (_b = (_a = this.options).onMessage) === null || _b === void 0 ? void 0 : _b.call(_a, {
                        routingKey: topic.replace(regex, ''),
                        messageContent: messageContent,
                        userId: (_c = topic.match(regex)) === null || _c === void 0 ? void 0 : _c[1]
                    });
                }
                if (topic.indexOf(this.DEVICE_ONLINE_PREFIX) > -1) {
                    const regex = new RegExp(`${this.DEVICE_ONLINE_PREFIX}/(.+?)/`);
                    let messageContent = JSON.parse((payload === null || payload === void 0 ? void 0 : payload.toString()) || '{error: "No message content"}');
                    const online_id = (_d = topic.match(regex)) === null || _d === void 0 ? void 0 : _d[1];
                    if (online_id && messageContent.offline != null) {
                        (_f = (_e = this.options).onStatus) === null || _f === void 0 ? void 0 : _f.call(_e, online_id, messageContent.offline ? 'OFFLINE' : 'ONLINE');
                    }
                }
            });
            this.client.on('reconnect', () => {
                console.log("MQTT Client reconnecting...");
            });
            this.client.on('error', (err) => {
                console.error(err);
            });
        });
    }
}
exports.MQTTHub = MQTTHub;
